=== modified file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	2006-01-16 06:56:15 +0000
+++ bzrlib/tests/test_selftest.py	2006-01-20 03:24:22 +0000
@@ -162,6 +162,7 @@
         local_abspath_test = test_iter.next()
         local_urlpath_test = test_iter.next()
         memory_test = test_iter.next()
+        readonly_test = test_iter.next()
         sftp_abs_test = test_iter.next()
         sftp_homedir_test = test_iter.next()
         sftp_sibling_abs_test = test_iter.next()

=== modified file 'bzrlib/tests/test_transport.py'
--- bzrlib/tests/test_transport.py	2006-01-10 05:45:16 +0000
+++ bzrlib/tests/test_transport.py	2006-01-20 03:24:22 +0000
@@ -25,6 +25,7 @@
 from bzrlib.tests import TestCase
 from bzrlib.transport import (_get_protocol_handlers,
                               _get_transport_modules,
+                              get_transport,
                               register_lazy_transport,
                               _set_protocol_handlers,
                               urlescape,
@@ -70,3 +71,31 @@
         self.assertEqual(True, transport.listable())
         self.assertEqual(False, transport.should_cache())
         self.assertEqual(False, transport.is_readonly())
+        
+        
+class ReadonlyDecoratorTransportTest(TestCase):
+    """Readonly decoration specific tests."""
+
+    def test_local_parameters(self):
+        import bzrlib.transport.readonly as readonly
+        # connect to . in readonly mode
+        transport = readonly.ReadonlyTransportDecorator('readonly+.')
+        self.assertEqual(True, transport.listable())
+        self.assertEqual(False, transport.should_cache())
+        self.assertEqual(True, transport.is_readonly())
+
+    def test_http_parameters(self):
+        import bzrlib.transport.readonly as readonly
+        from bzrlib.transport.http import HttpServer
+        # connect to . via http which is not listable
+        server = HttpServer()
+        server.setUp()
+        try:
+            transport = get_transport('readonly+' + server.get_url())
+            self.failUnless(isinstance(transport,
+                                       readonly.ReadonlyTransportDecorator))
+            self.assertEqual(False, transport.listable())
+            self.assertEqual(True, transport.should_cache())
+            self.assertEqual(True, transport.is_readonly())
+        finally:
+            server.tearDown()

=== modified file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	2006-01-12 01:02:32 +0000
+++ bzrlib/tests/test_transport_implementations.py	2006-01-20 03:24:22 +0000
@@ -491,6 +491,7 @@
 
         # Not much to do with a readonly transport
         if t.is_readonly():
+            self.assertRaises(TransportNotPossible, t.delete, 'missing')
             return
 
         t.put('a', StringIO('a little bit of text\n'))
@@ -646,17 +647,28 @@
 
         # SftpServer creates control files in the working directory
         # so lets move down a directory to avoid those.
-        t.mkdir('wd')
+        if not t.is_readonly():
+            t.mkdir('wd')
+        else:
+            os.mkdir('wd')
         t = t.clone('wd')
 
         self.assertEqual([], sorted_list(u'.'))
-        self.build_tree(['a', 'b', 'c/', 'c/d', 'c/e'], transport=t)
+        if not t.is_readonly():
+            self.build_tree(['a', 'b', 'c/', 'c/d', 'c/e'], transport=t)
+        else:
+            self.build_tree(['wd/a', 'wd/b', 'wd/c/', 'wd/c/d', 'wd/c/e'])
 
         self.assertEqual([u'a', u'b', u'c'], sorted_list(u'.'))
         self.assertEqual([u'd', u'e'], sorted_list(u'c'))
 
-        t.delete('c/d')
-        t.delete('b')
+        if not t.is_readonly():
+            t.delete('c/d')
+            t.delete('b')
+        else:
+            os.unlink('wd/c/d')
+            os.unlink('wd/b')
+            
         self.assertEqual([u'a', u'c'], sorted_list('.'))
         self.assertEqual([u'e'], sorted_list(u'c'))
 

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2006-01-12 01:02:32 +0000
+++ bzrlib/transport/__init__.py	2006-01-20 03:24:22 +0000
@@ -545,3 +545,4 @@
 register_lazy_transport('ftp://', 'bzrlib.transport.ftp', 'FtpTransport')
 register_lazy_transport('aftp://', 'bzrlib.transport.ftp', 'FtpTransport')
 register_lazy_transport('memory://', 'bzrlib.transport.memory', 'MemoryTransport')
+register_lazy_transport('readonly+', 'bzrlib.transport.readonly', 'ReadonlyTransportDecorator')

=== modified file 'bzrlib/transport/memory.py'
--- bzrlib/transport/memory.py	2006-01-10 05:25:43 +0000
+++ bzrlib/transport/memory.py	2006-01-20 03:24:22 +0000
@@ -38,7 +38,7 @@
 
 
 class MemoryTransport(Transport):
-    """This is the transport agent for local filesystem access."""
+    """This is an in memory file system for transient data storage."""
 
     def __init__(self, url=""):
         """Set the 'base' path where files will be stored."""

=== added file 'bzrlib/transport/readonly.py'
--- bzrlib/transport/readonly.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/readonly.py	2006-01-20 03:24:22 +0000
@@ -0,0 +1,133 @@
+# Copyright (C) 2006 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Implementation of Transport that adapts another transport to be readonly."""
+
+from bzrlib.errors import TransportNotPossible
+from bzrlib.transport import get_transport, Transport, Server
+
+
+class ReadonlyTransportDecorator(Transport):
+    """A decorator that can convert any transport to be readonly.
+    
+    This does not use __getattr__ hacks as we need to ensure that
+    new writable methods are overridden correctly.
+    """
+
+    def __init__(self, url, _decorated=None):
+        """Set the 'base' path where files will be stored.
+        
+        _decorated is a private parameter for cloning."""
+        assert url.startswith('readonly+')
+        decorated_url = url[len('readonly+'):]
+        if _decorated is None:
+            self._decorated = get_transport(decorated_url)
+        else:
+            self._decorated = _decorated
+        super(ReadonlyTransportDecorator, self).__init__(
+            "readonly+" + self._decorated.base)
+
+    def clone(self, offset=None):
+        """See Transport.clone()."""
+        decorated_clone = self._decorated.clone(offset)
+        return ReadonlyTransportDecorator("readonly+" + decorated_clone.base,
+                                          decorated_clone)
+
+    def abspath(self, relpath):
+        """See Transport.abspath()."""
+        return "readonly+" + self._decorated.abspath(relpath)
+
+    def append(self, relpath, f):
+        """See Transport.append()."""
+        raise TransportNotPossible('readonly transport')
+
+    def has(self, relpath):
+        """See Transport.has()."""
+        return self._decorated.has(relpath)
+
+    def delete(self, relpath):
+        """See Transport.delete()."""
+        raise TransportNotPossible('readonly transport')
+
+    def get(self, relpath):
+        """See Transport.get()."""
+        return self._decorated.get(relpath)
+
+    def put(self, relpath, f, mode=None):
+        """See Transport.put()."""
+        raise TransportNotPossible('readonly transport')
+
+    def mkdir(self, relpath, mode=None):
+        """See Transport.mkdir()."""
+        raise TransportNotPossible('readonly transport')
+
+    def is_readonly(self):
+        """See Transport.is_readonly."""
+        return True
+
+    def listable(self):
+        """See Transport.listable."""
+        return self._decorated.listable()
+
+    def iter_files_recursive(self):
+        """See Transport.iter_files_recursive()."""
+        return self._decorated.iter_files_recursive()
+    
+    def list_dir(self, relpath):
+        """See Transport.list_dir()."""
+        return self._decorated.list_dir(relpath)
+    
+    def should_cache(self):
+        """See Transport.should_cache()."""
+        return self._decorated.should_cache()
+
+    def stat(self, relpath):
+        """See Transport.stat()."""
+        return self._decorated.stat(relpath)
+
+#    def lock_read(self, relpath):
+#   TODO if needed / when tested
+#
+#    def lock_write(self, relpath):
+#   TODO if needed / when tested.
+
+
+class ReadonlyServer(Server):
+    """Server for the ReadonlyTransportDecorator for testing with."""
+
+    def setUp(self):
+        """See bzrlib.transport.Server.setUp."""
+        from bzrlib.transport.local import LocalRelpathServer
+        self._server = LocalRelpathServer()
+        self._server.setUp()
+
+    def tearDown(self):
+        """See bzrlib.transport.Server.tearDown."""
+        self._server.tearDown()
+
+    def get_bogus_url(self):
+        """See bzrlib.transport.Server.get_bogus_url."""
+        return "readonly+" + self._server.get_bogus_url()
+
+    def get_url(self):
+        """See bzrlib.transport.Server.get_url."""
+        return "readonly+" + self._server.get_url()
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    return [(ReadonlyTransportDecorator, ReadonlyServer),
+            ]

